`timescale 1 ns / 1 ps

module adc_in #
(
  parameter integer ADC_DATA_WIDTH = 14
)
(
  // System signals
  output wire                        adc_clk,

  // ADC signals
  input  wire                        adc_clk_p,
  input  wire                        adc_clk_n,
  input  wire [ADC_DATA_WIDTH-1:0]   adc_dat_a,
  input  wire [ADC_DATA_WIDTH-1:0]   adc_dat_b,
  
  output wire                        adc_csn,
  output wire                        adc_enc_n,
  output wire                        adc_enc_p,
  output wire [ADC_DATA_WIDTH-1:0]   adc_dat_a_o,
  output wire [ADC_DATA_WIDTH-1:0]   adc_dat_b_o
);

  reg  [ADC_DATA_WIDTH-1:0] int_dat_a_reg;
  reg  [ADC_DATA_WIDTH-1:0] int_dat_b_reg;
  wire                      int_clk0;
  wire 						int_clk;

  IBUFGDS adc_clk_inst0 (.I(adc_clk_p), .IB(adc_clk_n), .O(int_clk0));
  BUFG adc_clk_inst (.I(int_clk0), .O(int_clk));

  always @(posedge int_clk)
  begin
    int_dat_a_reg <= adc_dat_a;
    int_dat_b_reg <= adc_dat_b;
  end

  assign adc_dat_a_o = {int_dat_a_reg[ADC_DATA_WIDTH-1], ~int_dat_a_reg[ADC_DATA_WIDTH-2:0]}; // transform into 2's complement (negative slope)
  assign adc_dat_b_o = {int_dat_b_reg[ADC_DATA_WIDTH-1], ~int_dat_b_reg[ADC_DATA_WIDTH-2:0]};

  assign adc_clk = int_clk;

  assign adc_csn = 1'b1;
  
  assign adc_enc_n = 1'b0;
  assign adc_enc_p = 1'b1;

endmodule
